@@ -1,30 +1,35 @@
-#!/usr/bin/env python3
-# pylint: disable=missing-docstring,not-an-iterable,too-many-locals,too-many-arguments,invalid-name,too-many-return-statements,too-many-branches,len-as-condition,too-many-nested-blocks,wrong-import-order,duplicate-code, anomalous-backslash-in-string, too-many-statements, singleton-comparison, consider-using-in
-
-import singer
 import datetime
 import pytz
 import decimal
 import psycopg2
+from psycopg2 import sql
 import copy
 import json
+import singer
 import singer.metadata as metadata
-import tap_postgres.db as post_db
-import tap_postgres.sync_strategies.common as sync_common
 from singer import utils, get_bookmark
 from dateutil.parser import parse
 from functools import reduce
 
+import tap_postgres.db as post_db
+import tap_postgres.sync_strategies.common as sync_common
+
 LOGGER = singer.get_logger('tap_postgres')
 
 UPDATE_BOOKMARK_PERIOD = 10000
 
+
+class ReplicationSlotNotFoundError(Exception):
+    """Custom exception when replication slot not found"""
+
+
+# pylint: disable=invalid-name,missing-function-docstring,too-many-branches,too-many-statements,too-many-arguments
 def get_pg_version(conn_info):
     with post_db.open_connection(conn_info, False) as conn:
         with conn.cursor() as cur:
             cur.execute("SELECT setting::int AS version FROM pg_settings WHERE name='server_version_num'")
             version = cur.fetchone()[0]
-    LOGGER.debug("Detected PostgreSQL version: {}".format(version))
+    LOGGER.debug('Detected PostgreSQL version: %s', version)
     return version
 
 
@@ -35,8 +40,8 @@ def lsn_to_int(lsn):
         return None
 
     file, index = lsn.split('/')
-    lsni = (int(file, 16)  << 32) + int(index, 16)
-    return(lsni)
+    lsni = (int(file, 16) << 32) + int(index, 16)
+    return lsni
 
 
 def int_to_lsn(lsni):
@@ -58,24 +63,25 @@ def int_to_lsn(lsni):
     index = (format(int(lsnb[-32:], 2), 'x')).upper()
     # Formatting
     lsn = "{}/{}".format(file, index)
-    return(lsn)
+    return lsn
 
 
+# pylint: disable=chained-comparison
 def fetch_current_lsn(conn_config):
     version = get_pg_version(conn_config)
     # Make sure PostgreSQL version is 9.4 or higher
     # Do not allow minor versions with PostgreSQL BUG #15114
     if (version >= 110000) and (version < 110002):
         raise Exception('PostgreSQL upgrade required to minor version 11.2')
-    elif (version >= 100000) and (version < 100007):
+    if (version >= 100000) and (version < 100007):
         raise Exception('PostgreSQL upgrade required to minor version 10.7')
-    elif (version >= 90600) and (version < 90612):
+    if (version >= 90600) and (version < 90612):
         raise Exception('PostgreSQL upgrade required to minor version 9.6.12')
-    elif (version >= 90500) and (version < 90516):
+    if (version >= 90500) and (version < 90516):
         raise Exception('PostgreSQL upgrade required to minor version 9.5.16')
-    elif (version >= 90400) and (version < 90421):
+    if (version >= 90400) and (version < 90421):
         raise Exception('PostgreSQL upgrade required to minor version 9.4.21')
-    elif (version < 90400):
+    if version < 90400:
         raise Exception('Logical replication not supported before PostgreSQL 9.4')
 
     with post_db.open_connection(conn_config, False) as conn:
@@ -91,16 +97,18 @@ def fetch_current_lsn(conn_config):
             current_lsn = cur.fetchone()[0]
             return lsn_to_int(current_lsn)
 
+
 def add_automatic_properties(stream, conn_config):
-    stream['schema']['properties']['_sdc_deleted_at'] = {'type' : ['null', 'string'], 'format' :'date-time'}
+    stream['schema']['properties']['_sdc_deleted_at'] = {'type': ['null', 'string'], 'format': 'date-time'}
     if conn_config.get('debug_lsn'):
         LOGGER.debug('debug_lsn is ON')
-        stream['schema']['properties']['_sdc_lsn'] = {'type' : ['null', 'string']}
+        stream['schema']['properties']['_sdc_lsn'] = {'type': ['null', 'string']}
     else:
         LOGGER.debug('debug_lsn is OFF')
 
     return stream
 
+
 def get_stream_version(tap_stream_id, state):
     stream_version = singer.get_bookmark(state, tap_stream_id, 'version')
 
@@ -109,22 +117,26 @@ def get_stream_version(tap_stream_id, state):
 
     return stream_version
 
+
 def tuples_to_map(accum, t):
     accum[t[0]] = t[1]
     return accum
 
+
 def create_hstore_elem_query(elem):
-    return psycopg2.sql.SQL("SELECT hstore_to_array({})").format(psycopg2.sql.Literal(elem))
+    return sql.SQL("SELECT hstore_to_array({})").format(sql.Literal(elem))
+
 
 def create_hstore_elem(conn_info, elem):
     with post_db.open_connection(conn_info) as conn:
         with conn.cursor() as cur:
-            sql_stmt = create_hstore_elem_query(elem)
-            cur.execute(sql_stmt)
+            query = create_hstore_elem_query(elem)
+            cur.execute(query)
             res = cur.fetchone()[0]
             hstore_elem = reduce(tuples_to_map, [res[i:i + 2] for i in range(0, len(res), 2)], {})
             return hstore_elem
 
+
 def create_array_elem(elem, sql_datatype, conn_info):
     if elem is None:
         return None
@@ -175,7 +187,7 @@ def create_array_elem(elem, sql_datatype, conn_info):
                 cast_datatype = 'text[]'
 
             else:
-                #custom datatypes like enums
+                # custom datatypes like enums
                 cast_datatype = 'text[]'
 
             sql_stmt = """SELECT $stitch_quote${}$stitch_quote$::{}""".format(elem, cast_datatype)
@@ -183,7 +195,8 @@ def create_array_elem(elem, sql_datatype, conn_info):
             res = cur.fetchone()[0]
             return res
 
-#pylint: disable=too-many-branches,too-many-nested-blocks
+
+# pylint: disable=too-many-branches,too-many-nested-blocks,too-many-return-statements
 def selected_value_to_singer_value_impl(elem, og_sql_datatype, conn_info):
     sql_datatype = og_sql_datatype.replace('[]', '')
 
@@ -197,19 +210,20 @@ def selected_value_to_singer_value_impl(elem, og_sql_datatype, conn_info):
 
         return parse(elem).isoformat()
     if sql_datatype == 'date':
-        if  isinstance(elem, datetime.date):
-            #logical replication gives us dates as strings UNLESS they from an array
+        if isinstance(elem, datetime.date):
+            # logical replication gives us dates as strings UNLESS they from an array
             return elem.isoformat() + 'T00:00:00+00:00'
         return parse(elem).isoformat() + "+00:00"
     if sql_datatype == 'time with time zone':
-        '''time with time zone values will be converted to UTC and time zone dropped'''
+        # time with time zone values will be converted to UTC and time zone dropped
         # Replace hour=24 with hour=0
-        if elem.startswith('24'): elem = elem.replace('24','00',1)
+        if elem.startswith('24'):
+            elem = elem.replace('24', '00', 1)
         # convert to UTC
         elem = elem + '00'
         elem_obj = datetime.datetime.strptime(elem, '%H:%M:%S%z')
         if elem_obj.utcoffset() != datetime.timedelta(seconds=0):
-            LOGGER.warning('time with time zone values are converted to UTC'.format(og_sql_datatype))
+            LOGGER.warning('time with time zone values are converted to UTC: %s', og_sql_datatype)
         elem_obj = elem_obj.astimezone(pytz.utc)
         # drop time zone
         elem = elem_obj.strftime('%H:%M:%S')
@@ -217,12 +231,12 @@ def selected_value_to_singer_value_impl(elem, og_sql_datatype, conn_info):
     if sql_datatype == 'time without time zone':
         # Replace hour=24 with hour=0
         if elem.startswith('24'):
-            elem = elem.replace('24','00',1)
+            elem = elem.replace('24', '00', 1)
         return parse(elem).isoformat().split('T')[1]
     if sql_datatype == 'bit':
-        #for arrays, elem will == True
-        #for ordinary bits, elem will == '1'
-        return elem == '1' or elem == True
+        # for arrays, elem will == True
+        # for ordinary bits, elem will == '1'
+        return elem == '1' or elem is True
     if sql_datatype == 'boolean':
         return elem
     if sql_datatype == 'hstore':
@@ -238,24 +252,28 @@ def selected_value_to_singer_value_impl(elem, og_sql_datatype, conn_info):
 
     raise Exception("do not know how to marshall value of type {}".format(elem.__class__))
 
+
 def selected_array_to_singer_value(elem, sql_datatype, conn_info):
     if isinstance(elem, list):
         return list(map(lambda elem: selected_array_to_singer_value(elem, sql_datatype, conn_info), elem))
 
     return selected_value_to_singer_value_impl(elem, sql_datatype, conn_info)
 
+
 def selected_value_to_singer_value(elem, sql_datatype, conn_info):
-    #are we dealing with an array?
+    # are we dealing with an array?
     if sql_datatype.find('[]') > 0:
         cleaned_elem = create_array_elem(elem, sql_datatype, conn_info)
-        return list(map(lambda elem: selected_array_to_singer_value(elem, sql_datatype, conn_info), (cleaned_elem or [])))
+        return list(map(lambda elem: selected_array_to_singer_value(elem, sql_datatype, conn_info),
+                        (cleaned_elem or [])))
 
     return selected_value_to_singer_value_impl(elem, sql_datatype, conn_info)
 
+
 def row_to_singer_message(stream, row, version, columns, time_extracted, md_map, conn_info):
     row_to_persist = ()
-    md_map[('properties', '_sdc_deleted_at')] = {'sql-datatype' : 'timestamp with time zone'}
-    md_map[('properties', '_sdc_lsn')] = {'sql-datatype' : "character varying"}
+    md_map[('properties', '_sdc_deleted_at')] = {'sql-datatype': 'timestamp with time zone'}
+    md_map[('properties', '_sdc_lsn')] = {'sql-datatype': "character varying"}
 
     for idx, elem in enumerate(row):
         sql_datatype = md_map.get(('properties', columns[idx])).get('sql-datatype')
@@ -275,11 +293,13 @@ def row_to_singer_message(stream, row, version, columns, time_extracted, md_map,
         version=version,
         time_extracted=time_extracted)
 
+
+# pylint: disable=unused-argument,too-many-locals
 def consume_message(streams, state, msg, time_extracted, conn_info, end_lsn):
     # Strip leading comma generated by write-in-chunks and parse valid JSON
     try:
         payload = json.loads(msg.payload.lstrip(','))
-    except:
+    except Exception:
         return state
 
     lsn = msg.data_start
@@ -296,7 +316,8 @@ def consume_message(streams, state, msg, time_extracted, conn_info, end_lsn):
     stream_version = get_stream_version(target_stream['tap_stream_id'], state)
     stream_md_map = metadata.to_map(target_stream['metadata'])
 
-    desired_columns = [c for c in target_stream['schema']['properties'].keys() if sync_common.should_sync_column(stream_md_map, c)]
+    desired_columns = [c for c in target_stream['schema']['properties'].keys() if sync_common.should_sync_column(
+        stream_md_map, c)]
 
     if payload['kind'] == 'insert':
         col_names = []
@@ -311,7 +332,13 @@ def consume_message(streams, state, msg, time_extracted, conn_info, end_lsn):
         if conn_info.get('debug_lsn'):
             col_names = col_names + ['_sdc_lsn']
             col_vals = col_vals + [str(lsn)]
-        record_message = row_to_singer_message(target_stream, col_vals, stream_version, col_names, time_extracted, stream_md_map, conn_info)
+        record_message = row_to_singer_message(target_stream,
+                                               col_vals,
+                                               stream_version,
+                                               col_names,
+                                               time_extracted,
+                                               stream_md_map,
+                                               conn_info)
 
     elif payload['kind'] == 'update':
         col_names = []
@@ -327,7 +354,13 @@ def consume_message(streams, state, msg, time_extracted, conn_info, end_lsn):
         if conn_info.get('debug_lsn'):
             col_vals = col_vals + [str(lsn)]
             col_names = col_names + ['_sdc_lsn']
-        record_message = row_to_singer_message(target_stream, col_vals, stream_version, col_names, time_extracted, stream_md_map, conn_info)
+        record_message = row_to_singer_message(target_stream,
+                                               col_vals,
+                                               stream_version,
+                                               col_names,
+                                               time_extracted,
+                                               stream_md_map,
+                                               conn_info)
 
     elif payload['kind'] == 'delete':
         col_names = []
@@ -338,11 +371,17 @@ def consume_message(streams, state, msg, time_extracted, conn_info, end_lsn):
                 col_vals.append(payload['oldkeys']['keyvalues'][idx])
 
         col_names = col_names + ['_sdc_deleted_at']
-        col_vals = col_vals  + [singer.utils.strftime(time_extracted)]
+        col_vals = col_vals + [singer.utils.strftime(time_extracted)]
         if conn_info.get('debug_lsn'):
             col_vals = col_vals + [str(lsn)]
             col_names = col_names + ['_sdc_lsn']
-        record_message = row_to_singer_message(target_stream, col_vals, stream_version, col_names, time_extracted, stream_md_map, conn_info)
+        record_message = row_to_singer_message(target_stream,
+                                               col_vals,
+                                               stream_version,
+                                               col_names,
+                                               time_extracted,
+                                               stream_md_map,
+                                               conn_info)
 
     else:
         raise Exception("unrecognized replication operation: {}".format(payload['kind']))
@@ -352,16 +391,78 @@ def consume_message(streams, state, msg, time_extracted, conn_info, end_lsn):
 
     return state
 
+
+def generate_replication_slot_name(dbname, tap_id=None, prefix='pipelinewise'):
+    """Generate replication slot name with
+
+    :param str dbname: Database name that will be part of the replication slot name
+    :param str tap_id: Optional. If provided then it will be appended to the end of the slot name
+    :param str prefix: Optional. Defaults to 'pipelinewise'
+    :return: well formatted lowercased replication slot name
+    :rtype: str
+    """
+    # Add tap_id to the end of the slot name if provided
+    if tap_id:
+        tap_id = f'_{tap_id}'
+    # Convert None to empty string
+    else:
+        tap_id = ''
+    return f'{prefix}_{dbname}{tap_id}'.lower()
+
+
+def locate_replication_slot_by_cur(cursor, dbname, tap_id=None):
+    slot_name_v15 = generate_replication_slot_name(dbname)
+    slot_name_v16 = generate_replication_slot_name(dbname, tap_id)
+
+    # Backward compatibility: try to locate existing v15 slot first. PPW <= 0.15.0
+    cursor.execute(f"SELECT * FROM pg_replication_slots WHERE slot_name = '{slot_name_v15}'")
+    if len(cursor.fetchall()) == 1:
+        LOGGER.info('Using pg_replication_slot %s', slot_name_v15)
+        return slot_name_v15
+
+    # v15 style replication slot not found, try to locate v16
+    cursor.execute(f"SELECT * FROM pg_replication_slots WHERE slot_name = '{slot_name_v16}'")
+    if len(cursor.fetchall()) == 1:
+        LOGGER.info('Using pg_replication_slot %s', slot_name_v16)
+        return slot_name_v16
+
+    raise ReplicationSlotNotFoundError(f'Unable to find replication slot {slot_name_v16}')
+
+
 def locate_replication_slot(conn_info):
     with post_db.open_connection(conn_info, False) as conn:
         with conn.cursor() as cur:
-            db_specific_slot = "pipelinewise_{}".format(conn_info['dbname'].lower())
-            cur.execute("SELECT * FROM pg_replication_slots WHERE slot_name = %s AND plugin = %s", (db_specific_slot, 'wal2json'))
-            if len(cur.fetchall()) == 1:
-                LOGGER.info("Using pg_replication_slot %s", db_specific_slot)
-                return db_specific_slot
-            else:
-                raise Exception("Unable to find replication slot {} with wal2json".format(db_specific_slot))
+            return locate_replication_slot_by_cur(cur, conn_info['dbname'], conn_info['tap_id'])
+
+
+# pylint: disable=anomalous-backslash-in-string
+def streams_to_wal2json_tables(streams):
+    """Converts a list of singer stream dictionaries to wal2json plugin compatible string list.
+    The output is compatible with the 'filter-tables' and 'add-tables' option of wal2json plugin.
+
+    Special characters (space, single quote, comma, period, asterisk) must be escaped with backslash.
+    Schema and table are case-sensitive. Table "public"."Foo bar" should be specified as "public.Foo\ bar".
+    Documentation in wal2json plugin: https://github.com/eulerto/wal2json/blob/master/README.md#parameters
+
+    :param streams: List of singer stream dictionaries
+    :return: tables(str): comma separated and escaped list of tables, compatible for wal2json plugin
+    :rtype: str
+    """
+    def escape_spec_chars(string):
+        escaped = string
+        wal2json_special_chars = " ',.*"
+        for ch in wal2json_special_chars:
+            escaped = escaped.replace(ch, f'\\{ch}')
+        return escaped
+
+    tables = []
+    for s in streams:
+        schema_name = escape_spec_chars(s['metadata'][0]['metadata']['schema-name'])
+        table_name = escape_spec_chars(s['table_name'])
+
+        tables.append(f'{schema_name}.{table_name}')
+
+    return ','.join(tables)
 
 
 def sync_tables(conn_info, logical_streams, state, end_lsn, state_file):
@@ -378,14 +479,10 @@ def sync_tables(conn_info, logical_streams, state, end_lsn, state_file):
     start_run_timestamp = datetime.datetime.utcnow()
     max_run_seconds = conn_info['max_run_seconds']
     break_at_end_lsn = conn_info['break_at_end_lsn']
-    logical_poll_total_seconds = conn_info['logical_poll_total_seconds'] or 10800 #3 hours
+    logical_poll_total_seconds = conn_info['logical_poll_total_seconds'] or 10800  # 3 hours
     poll_interval = 10
     poll_timestamp = None
 
-    selected_tables = []
-    for s in logical_streams:
-        selected_tables.append("{}.{}".format(s['metadata'][0]['metadata']['schema-name'], s['table_name']))
-
     for s in logical_streams:
         sync_common.send_schema_message(s, ['lsn'])
 
@@ -396,17 +493,28 @@ def sync_tables(conn_info, logical_streams, state, end_lsn, state_file):
     cur = conn.cursor()
 
     # Set session wal_sender_timeout for PG12 and above
-    if (version >= 120000):
-        wal_sender_timeout = 10800000 #10800000ms = 3 hours
-        LOGGER.info("Set session wal_sender_timeout = {} milliseconds".format(wal_sender_timeout))
+    if version >= 120000:
+        wal_sender_timeout = 10800000  # 10800000ms = 3 hours
+        LOGGER.info('Set session wal_sender_timeout = %i milliseconds', wal_sender_timeout)
         cur.execute("SET SESSION wal_sender_timeout = {}".format(wal_sender_timeout))
 
     try:
-        LOGGER.info("Request wal streaming from {} to {} (slot {})".format(int_to_lsn(start_lsn), int_to_lsn(end_lsn), slot))
+        LOGGER.info('Request wal streaming from %s to %s (slot %s)',
+                    int_to_lsn(start_lsn),
+                    int_to_lsn(end_lsn),
+                    slot)
         # psycopg2 2.8.4 will send a keep-alive message to postgres every status_interval
-        cur.start_replication(slot_name=slot, decode=True, start_lsn=start_lsn, status_interval=poll_interval, options={'write-in-chunks': 1, 'add-tables': ','.join(selected_tables)})
-    except psycopg2.ProgrammingError:
-        raise Exception("Unable to start replication with logical replication (slot {})".format(slot))
+        cur.start_replication(slot_name=slot,
+                              decode=True,
+                              start_lsn=start_lsn,
+                              status_interval=poll_interval,
+                              options={
+                                  'write-in-chunks': 1,
+                                  'add-tables': streams_to_wal2json_tables(logical_streams)
+                              })
+
+    except psycopg2.ProgrammingError as ex:
+        raise Exception("Unable to start replication with logical replication (slot {})".format(ex))
 
     lsn_received_timestamp = datetime.datetime.utcnow()
     poll_timestamp = datetime.datetime.utcnow()
@@ -416,22 +524,24 @@ def sync_tables(conn_info, logical_streams, state, end_lsn, state_file):
         # needs to be long enough to wait for the largest single wal payload to avoid unplanned timeouts
         poll_duration = (datetime.datetime.utcnow() - lsn_received_timestamp).total_seconds()
         if poll_duration > logical_poll_total_seconds:
-            LOGGER.info("Breaking - {} seconds of polling with no data".format(poll_duration))
+            LOGGER.info('Breaking - %i seconds of polling with no data', poll_duration)
             break
 
         try:
             msg = cur.read_message()
         except Exception as e:
-            LOGGER.error("{}".format(e))
+            LOGGER.error(e)
             raise
 
         if msg:
             if (break_at_end_lsn) and (msg.data_start > end_lsn):
-                LOGGER.info("Breaking - latest wal message {} is past end_lsn {}".format(int_to_lsn(msg.data_start), int_to_lsn(end_lsn)))
+                LOGGER.info('Breaking - latest wal message %s is past end_lsn %s',
+                            int_to_lsn(msg.data_start),
+                            int_to_lsn(end_lsn))
                 break
 
             if datetime.datetime.utcnow() >= (start_run_timestamp + datetime.timedelta(seconds=max_run_seconds)):
-                LOGGER.info("Breaking - reached max_run_seconds of {}".format(max_run_seconds))
+                LOGGER.info('Breaking - reached max_run_seconds of %i', max_run_seconds)
                 break
 
             state = consume_message(logical_streams, state, msg, time_extracted, conn_info, end_lsn)
@@ -440,21 +550,26 @@ def sync_tables(conn_info, logical_streams, state, end_lsn, state_file):
             # This is to ensure we only flush to lsn that has completed entirely
             if lsn_currently_processing is None:
                 lsn_currently_processing = msg.data_start
-                LOGGER.info("First wal message received is {}".format(int_to_lsn(lsn_currently_processing)))
+                LOGGER.info('First wal message received is %s', int_to_lsn(lsn_currently_processing))
 
                 # Flush Postgres wal up to lsn comitted in previous run, or first lsn received in this run
                 lsn_to_flush = lsn_comitted
-                if lsn_currently_processing < lsn_to_flush: lsn_to_flush = lsn_currently_processing
-                LOGGER.info("Confirming write up to {}, flush to {}".format(int_to_lsn(lsn_to_flush), int_to_lsn(lsn_to_flush)))
+                if lsn_currently_processing < lsn_to_flush:
+                    lsn_to_flush = lsn_currently_processing
+                LOGGER.info('Confirming write up to %s, flush to %s',
+                            int_to_lsn(lsn_to_flush),
+                            int_to_lsn(lsn_to_flush))
                 cur.send_feedback(write_lsn=lsn_to_flush, flush_lsn=lsn_to_flush, reply=True, force=True)
 
-            elif (int(msg.data_start) > lsn_currently_processing):
+            elif int(msg.data_start) > lsn_currently_processing:
                 lsn_last_processed = lsn_currently_processing
                 lsn_currently_processing = msg.data_start
                 lsn_received_timestamp = datetime.datetime.utcnow()
                 lsn_processed_count = lsn_processed_count + 1
                 if lsn_processed_count >= UPDATE_BOOKMARK_PERIOD:
-                    LOGGER.debug("Updating bookmarks for all streams to lsn = {} ({})".format(lsn_last_processed, int_to_lsn(lsn_last_processed)))
+                    LOGGER.debug('Updating bookmarks for all streams to lsn = %s (%s)',
+                                 lsn_last_processed,
+                                 int_to_lsn(lsn_last_processed))
                     for s in logical_streams:
                         state = singer.write_bookmark(state, s['tap_stream_id'], 'lsn', lsn_last_processed)
                     singer.write_message(singer.StateMessage(value=copy.deepcopy(state)))
@@ -463,19 +578,22 @@ def sync_tables(conn_info, logical_streams, state, end_lsn, state_file):
         # Every poll_interval, update latest comitted lsn position from the state_file
         if datetime.datetime.utcnow() >= (poll_timestamp + datetime.timedelta(seconds=poll_interval)):
             if lsn_currently_processing is None:
-                LOGGER.info("Waiting for first wal message")
+                LOGGER.info('Waiting for first wal message')
             else:
-                LOGGER.info("Lastest wal message received was {}".format(int_to_lsn(lsn_last_processed)))
+                LOGGER.info('Lastest wal message received was %s', int_to_lsn(lsn_last_processed))
                 try:
                     state_comitted_file = open(state_file)
                     state_comitted = json.load(state_comitted_file)
-                except:
-                    LOGGER.debug("Unable to open and parse {}".format(state_file))
+                except Exception:
+                    LOGGER.debug('Unable to open and parse %s', state_file)
                 finally:
-                    lsn_comitted = min([get_bookmark(state_comitted, s['tap_stream_id'], 'lsn') for s in logical_streams])
+                    lsn_comitted = min(
+                        [get_bookmark(state_comitted, s['tap_stream_id'], 'lsn') for s in logical_streams])
                     if (lsn_currently_processing > lsn_comitted) and (lsn_comitted > lsn_to_flush):
                         lsn_to_flush = lsn_comitted
-                        LOGGER.info("Confirming write up to {}, flush to {}".format(int_to_lsn(lsn_to_flush), int_to_lsn(lsn_to_flush)))
+                        LOGGER.info('Confirming write up to %s, flush to %s',
+                                    int_to_lsn(lsn_to_flush),
+                                    int_to_lsn(lsn_to_flush))
                         cur.send_feedback(write_lsn=lsn_to_flush, flush_lsn=lsn_to_flush, reply=True, force=True)
 
             poll_timestamp = datetime.datetime.utcnow()
@@ -487,9 +605,14 @@ def sync_tables(conn_info, logical_streams, state, end_lsn, state_file):
     if lsn_last_processed:
         if lsn_comitted > lsn_last_processed:
             lsn_last_processed = lsn_comitted
-            LOGGER.info("Current lsn_last_processed {} is older than lsn_comitted {}".format(int_to_lsn(lsn_last_processed), int_to_lsn(lsn_comitted)))
+            LOGGER.info('Current lsn_last_processed %s is older than lsn_comitted %s',
+                        int_to_lsn(lsn_last_processed),
+                        int_to_lsn(lsn_comitted))
+
+        LOGGER.info('Updating bookmarks for all streams to lsn = %s (%s)',
+                    lsn_last_processed,
+                    int_to_lsn(lsn_last_processed))
 
-        LOGGER.info("Updating bookmarks for all streams to lsn = {} ({})".format(lsn_last_processed, int_to_lsn(lsn_last_processed)))
         for s in logical_streams:
             state = singer.write_bookmark(state, s['tap_stream_id'], 'lsn', lsn_last_processed)
 
